@@ -77,6 +77,8 @@ web_radiobutton_toggled_cb (GtkWidget *togglebutton, GnomeDACapplet *capplet)
 	return;
 
     item = (GnomeDAWebItem *) g_list_nth_data (capplet->web_browsers, index);
+    if (item == NULL)
+        return;
 
     if (togglebutton == capplet->new_win_radiobutton) {
 	command = item->win_command;
@@ -90,6 +92,8 @@ web_radiobutton_toggled_cb (GtkWidget *togglebutton, GnomeDACapplet *capplet)
 
     gconf_client_set_string (capplet->gconf, DEFAULT_APPS_KEY_HTTP_EXEC, command, &error);
 
+    gtk_entry_set_text (GTK_ENTRY (capplet->web_browser_command_entry), command);
+
     if (error != NULL) {
 	g_warning (_("Error saving configuration: %s"), error->message);
 	g_error_free (error);
@@ -103,6 +107,7 @@ web_combo_changed_cb (GtkComboBox *combo, GnomeDACapplet *capplet)
     gboolean is_custom_active;
     gboolean has_net_remote;
     GnomeDAWebItem *item;
+    GtkWidget *active = NULL;
 
     current_index = gtk_combo_box_get_active (combo);
 
@@ -124,6 +129,18 @@ web_combo_changed_cb (GtkComboBox *combo, GnomeDACapplet *capplet)
     gtk_widget_set_sensitive (capplet->web_browser_command_entry, is_custom_active);
     gtk_widget_set_sensitive (capplet->web_browser_command_label, is_custom_active);
     gtk_widget_set_sensitive (capplet->web_browser_terminal_checkbutton, is_custom_active);
+
+    if (has_net_remote) {
+
+        if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (capplet->new_win_radiobutton)))
+            active = capplet->new_win_radiobutton;
+        else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (capplet->new_tab_radiobutton)))
+            active = capplet->new_tab_radiobutton;
+        else
+            active = capplet->default_radiobutton;
+    }
+
+    web_radiobutton_toggled_cb (active, capplet);
 }
 
 /* FIXME: Refactor these two functions below into one... */
@@ -450,6 +467,7 @@ web_combo_conv_to_widget (GConfPropertyEditor *peditor, const GConfValue *value)
 
     ret = gconf_value_new (GCONF_VALUE_INT);
     gconf_value_set_int (ret, index);
+
     return ret;
 }
 
