@@ -49,6 +49,8 @@ struct App
     GtkWidget	   *dialog;
     GtkWidget      *current_monitor_event_box;
     GtkWidget      *current_monitor_label;
+    GtkWidget      *monitor_on_radio;
+    GtkWidget      *monitor_off_radio;
     GtkListStore   *resolution_store;
     GtkWidget	   *resolution_combo;
     GtkWidget	   *refresh_combo;
@@ -60,12 +62,19 @@ struct App
     GtkWidget      *area;
     gboolean	    ignore_gui_changes;
     GConfClient	   *client;
+
+    /* These are used while we are waiting for the ApplyConfiguration method to be executed over D-bus */
+    DBusGConnection *connection;
+    DBusGProxy *proxy;
+    DBusGProxyCall *proxy_call;
 };
 
 static void rebuild_gui (App *app);
 static void on_rate_changed (GtkComboBox *box, gpointer data);
 static gboolean output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config);
 static void select_current_output_from_dialog_position (App *app);
+static void monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data);
+static void get_geometry (GnomeOutputInfo *output, int *w, int *h);
 
 static void
 error_message (App *app, const char *primary_text, const char *secondary_text)
@@ -73,7 +82,7 @@ error_message (App *app, const char *primary_text, const char *secondary_text)
     GtkWidget *dialog;
 
     dialog = gtk_message_dialog_new ((app && app->dialog) ? GTK_WINDOW (app->dialog) : NULL,
-				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_MESSAGE_ERROR,
 				     GTK_BUTTONS_CLOSE,
 				     "%s", primary_text);
@@ -121,6 +130,7 @@ on_screen_changed (GnomeRRScreen *scr,
 	gnome_rr_config_free (app->current_configuration);
 
     app->current_configuration = current;
+    app->current_output = NULL;
 
 #if 0
     for (i = 0; app->current_configuration->outputs[i] != NULL; ++i)
@@ -344,17 +354,17 @@ rebuild_rotation_combo (App *app)
 	/* NULL-GError --- FIXME: we should say why this rotation is not available! */
 	if (gnome_rr_config_applicable (app->current_configuration, app->screen, NULL))
 	{
- 	    add_key (app->rotation_combo, info->name, 0, 0, 0, info->rotation);
+ 	    add_key (app->rotation_combo, _(info->name), 0, 0, 0, info->rotation);
 
 	    if (info->rotation == current)
-		selection = info->name;
+		selection = _(info->name);
 	}
     }
 
     app->current_output->rotation = current;
 
     if (!(selection && combo_select (app->rotation_combo, selection)))
-	combo_select (app->rotation_combo, N_("Normal"));
+	combo_select (app->rotation_combo, _("Normal"));
 }
 
 static void
@@ -370,11 +380,12 @@ rebuild_rate_combo (App *app)
     gtk_widget_set_sensitive (
 	app->refresh_combo, app->current_output && app->current_output->on);
 
-    if (!(modes = get_current_modes (app)))
+    if (!app->current_output
+        || !(modes = get_current_modes (app)))
 	return;
 
     rates = g_hash_table_new_full (
-	g_str_hash, g_str_equal, (GFreeFunc)g_free, NULL);
+	g_str_hash, g_str_equal, (GFreeFunc) g_free, NULL);
 
     best = -1;
     for (i = 0; modes[i] != NULL; ++i)
@@ -463,6 +474,43 @@ rebuild_current_monitor_label (App *app)
 	gtk_event_box_set_visible_window (GTK_EVENT_BOX (app->current_monitor_event_box), use_color);
 }
 
+static void
+rebuild_on_off_radios (App *app)
+{
+    gboolean sensitive;
+    gboolean on_active;
+    gboolean off_active;
+
+    g_signal_handlers_block_by_func (app->monitor_on_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
+    g_signal_handlers_block_by_func (app->monitor_off_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
+
+    if (app->current_output)
+    {
+	if (count_active_outputs (app) > 1 || !app->current_output->on)
+	    sensitive = TRUE;
+	else
+	    sensitive = FALSE;
+
+	on_active = app->current_output->on;
+	off_active = !on_active;
+    }
+    else
+    {
+	sensitive = FALSE;
+	on_active = FALSE;
+	off_active = FALSE;
+    }
+
+    gtk_widget_set_sensitive (app->monitor_on_radio, sensitive);
+    gtk_widget_set_sensitive (app->monitor_off_radio, sensitive);
+
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->monitor_on_radio), on_active);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->monitor_off_radio), off_active);
+
+    g_signal_handlers_unblock_by_func (app->monitor_on_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
+    g_signal_handlers_unblock_by_func (app->monitor_off_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
+}
+
 static void
 rebuild_resolution_combo (App *app)
 {
@@ -473,8 +521,17 @@ rebuild_resolution_combo (App *app)
 
     clear_combo (app->resolution_combo);
 
-    if (!(modes = get_current_modes (app)))
+    if (!(modes = get_current_modes (app))
+	|| !app->current_output
+	|| !app->current_output->on)
+    {
+	gtk_widget_set_sensitive (app->resolution_combo, FALSE);
 	return;
+    }
+
+    g_assert (app->current_output != NULL);
+
+    gtk_widget_set_sensitive (app->resolution_combo, TRUE);
 
     best_w = 0;
     best_h = 0;
@@ -496,20 +553,9 @@ rebuild_resolution_combo (App *app)
 	}
     }
 
-    if (count_active_outputs (app) > 1 || !app->current_output->on)
-	add_key (app->resolution_combo, _("Off"), 0, 0, 0, 0);
-
-    if (!app->current_output->on)
-    {
-	current = "Off";
-    }
-    else
-    {
-	current = idle_free (g_strdup_printf (_("%d x %d"),
-					      app->current_output->width,
-					      app->current_output->height));
-    }
-
+    current = idle_free (g_strdup_printf (_("%d x %d"),
+					  app->current_output->width,
+					  app->current_output->height));
 
     if (!combo_select (app->resolution_combo, current))
     {
@@ -531,19 +577,18 @@ rebuild_gui (App *app)
 
     app->ignore_gui_changes = TRUE;
 
-    sensitive = app->current_output? TRUE : FALSE;
+    sensitive = app->current_output ? TRUE : FALSE;
 
 #if 0
     g_debug ("rebuild gui, is on: %d", app->current_output->on);
 #endif
 
     rebuild_current_monitor_label (app);
+    rebuild_on_off_radios (app);
     rebuild_resolution_combo (app);
     rebuild_rate_combo (app);
     rebuild_rotation_combo (app);
 
-    gtk_widget_set_sensitive (app->resolution_combo, sensitive);
-
 #if 0
     g_debug ("sensitive: %d, on: %d", sensitive, app->current_output->on);
 #endif
@@ -622,16 +667,92 @@ on_rate_changed (GtkComboBox *box, gpointer data)
     foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
 }
 
+static void
+monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data)
+{
+    App *app = data;
+    gboolean is_on;
+
+    if (!app->current_output)
+	return;
+
+    if (!gtk_toggle_button_get_active (toggle))
+	return;
+
+    if (GTK_WIDGET (toggle) == app->monitor_on_radio)
+	is_on = TRUE;
+    else if (GTK_WIDGET (toggle) == app->monitor_off_radio)
+	is_on = FALSE;
+    else
+    {
+	g_assert_not_reached ();
+	return;
+    }
+
+    app->current_output->on = is_on;
+
+    rebuild_gui (app);
+    foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
+}
+
+static void
+realign_outputs_after_resolution_change (App *app, GnomeOutputInfo *output_that_changed, int old_width, int old_height)
+{
+    /* We find the outputs that were below or to the right of the output that
+     * changed, and realign them; we also do that for outputs that shared the
+     * right/bottom edges with the output that changed.  The outputs that are
+     * above or to the left of that output don't need to change.
+     */
+
+    int i;
+    int old_right_edge, old_bottom_edge;
+    int dx, dy;
+
+    g_assert (app->current_configuration != NULL);
+
+    old_right_edge = output_that_changed->x + old_width;
+    old_bottom_edge = output_that_changed->y + old_height;
+
+    dx = output_that_changed->width - old_width;
+    dy = output_that_changed->height - old_height;
+
+    for (i = 0; app->current_configuration->outputs[i] != NULL; i++) {
+	GnomeOutputInfo *output;
+	int output_width, output_height;
+
+	output = app->current_configuration->outputs[i];
+
+	if (output == output_that_changed || !output->connected)
+	    continue;
+
+	get_geometry (output, &output_width, &output_height);
+
+	if (output->x >= old_right_edge)
+	    output->x += dx;
+	else if (output->x + output_width == old_right_edge)
+	    output->x = output_that_changed->x + output_that_changed->width - output_width;
+
+	if (output->y >= old_bottom_edge)
+	    output->y += dy;
+	else if (output->y + output_height == old_bottom_edge)
+	    output->y = output_that_changed->y + output_that_changed->height - output_height;
+    }
+}
+
 static void
 on_resolution_changed (GtkComboBox *box, gpointer data)
 {
     App *app = data;
+    int old_width, old_height;
     int width;
     int height;
 
     if (!app->current_output)
 	return;
 
+    old_width = app->current_output->width;
+    old_height = app->current_output->height;
+
     if (get_mode (app->resolution_combo, &width, &height, NULL, NULL))
     {
 	app->current_output->width = width;
@@ -643,23 +764,7 @@ on_resolution_changed (GtkComboBox *box, gpointer data)
 	    app->current_output->on = TRUE;
     }
 
-#if 0
-    if (app->current_configuration)
-    {
-	x = 0;
-	for (i = 0; app->current_configuration->outputs[i] != NULL; ++i)
-	{
-	    GnomeOutputInfo *output = app->current_configuration->outputs[i];
-
-	    if (output->connected)
-	    {
-		output->x = x;
-
-		x += output->width;
-	    }
-	}
-    }
-#endif
+    realign_outputs_after_resolution_change (app, app->current_output, old_width, old_height);
 
     rebuild_rate_combo (app);
     rebuild_rotation_combo (app);
@@ -705,7 +810,7 @@ lay_out_outputs_horizontally (App *app)
 
 	x += output->width;
     }
-    
+
 }
 
 static void
@@ -1475,7 +1580,7 @@ paint_output (App *app, cairo_t *cr, int i)
     cairo_move_to (cr, extent.x, extent.y);
 
     if (output->on)
-	cairo_set_source_rgb (cr, 0.2, 0.2, 0.8);
+	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
     else
 	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 
@@ -1606,12 +1711,40 @@ check_required_virtual_size (App *app)
     }
 }
 
+/* Callback for dbus_g_proxy_begin_call() */
+static void
+apply_configuration_returned_cb (DBusGProxy       *proxy,
+				 DBusGProxyCall   *call_id,
+				 void             *data)
+{
+    App *app = data;
+    gboolean success;
+    GError *error;
+
+    g_assert (call_id == app->proxy_call);
+
+    error = NULL;
+    success = dbus_g_proxy_end_call (proxy, call_id, &error, G_TYPE_INVALID);
+
+    if (!success) {
+	error_message (app, _("Could not apply the selected configuration"), error->message);
+	g_error_free (error);
+    }
+
+    g_object_unref (app->proxy);
+    app->proxy = NULL;
+
+    dbus_g_connection_unref (app->connection);
+    app->connection = NULL;
+    app->proxy_call = NULL;
+
+    gtk_widget_set_sensitive (app->dialog, TRUE);
+}
+
 static void
 apply (App *app)
 {
     GError *error = NULL;
-    DBusGConnection *connection;
-    DBusGProxy *proxy;
 
     gnome_rr_config_sanitize (app->current_configuration);
 
@@ -1626,30 +1759,34 @@ apply (App *app)
 	return;
     }
 
-    connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-    if (connection == NULL) {
+    g_assert (app->connection == NULL);
+    g_assert (app->proxy == NULL);
+    g_assert (app->proxy_call == NULL);
+
+    app->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+    if (app->connection == NULL) {
 	error_message (app, _("Could not get session bus while applying display configuration"), error->message);
 	g_error_free (error);
 	return;
     }
 
-    proxy = dbus_g_proxy_new_for_name (connection,
-				       "org.gnome.SettingsDaemon",
-				       "/org/gnome/SettingsDaemon/XRANDR",
-				       "org.gnome.SettingsDaemon.XRANDR");
-    if (!proxy) {
+    app->proxy = dbus_g_proxy_new_for_name (app->connection,
+					    "org.gnome.SettingsDaemon",
+					    "/org/gnome/SettingsDaemon/XRANDR",
+					    "org.gnome.SettingsDaemon.XRANDR");
+    if (!app->proxy) {
 	error_message (app, _("Could not get org.gnome.SettingsDaemon.XRANDR"), NULL);
+	dbus_g_connection_unref (app->connection);
+	app->connection = NULL;
 	return;
-
     }
 
-    if (!dbus_g_proxy_call (proxy, "ApplyConfiguration", &error, G_TYPE_INVALID, G_TYPE_INVALID)) {
-	error_message (app, _("Could not apply the selected configuration"), error->message);
-	g_error_free (error);
-    }
-
-    g_object_unref (proxy);
-    dbus_g_connection_unref (connection);
+    gtk_widget_set_sensitive (app->dialog, FALSE);
+    app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
+					       apply_configuration_returned_cb, app,
+					       NULL,
+					       G_TYPE_INVALID,
+					       G_TYPE_INVALID);
 }
 
 #if 0
@@ -1748,7 +1885,7 @@ get_nearest_output (GnomeRRConfig *configuration, int x, int y)
 	return configuration->outputs[nearest_index];
     else
 	return NULL;
-		
+
 }
 
 /* Gets the output that contains the largest intersection with the window.
@@ -1780,7 +1917,7 @@ get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
 	output_rect.width  = output->width;
 	output_rect.height = output->height;
 
-	if (output->connected && output->on && gdk_rectangle_intersect (&win_rect, &output_rect, &intersection))
+	if (output->connected && gdk_rectangle_intersect (&win_rect, &output_rect, &intersection))
 	{
 	    int area;
 
@@ -1807,15 +1944,10 @@ get_output_for_window (GnomeRRConfig *configuration, GdkWindow *window)
 static void
 select_current_output_from_dialog_position (App *app)
 {
-    GnomeRROutput *output;
-
     if (GTK_WIDGET_REALIZED (app->dialog))
-    {
-	output = get_output_for_window (app->current_configuration, app->dialog->window);
-
-	if (output)
-	    app->current_output = output;
-    }
+	app->current_output = get_output_for_window (app->current_configuration, app->dialog->window);
+    else
+	app->current_output = NULL;
 
     rebuild_gui (app);
 }
@@ -1901,6 +2033,13 @@ run_application (App *app)
     app->current_monitor_event_box = glade_xml_get_widget (xml, "current_monitor_event_box");
     app->current_monitor_label = glade_xml_get_widget (xml, "current_monitor_label");
 
+    app->monitor_on_radio = glade_xml_get_widget (xml, "monitor_on_radio");
+    app->monitor_off_radio = glade_xml_get_widget (xml, "monitor_off_radio");
+    g_signal_connect (app->monitor_on_radio, "toggled",
+		      G_CALLBACK (monitor_on_off_toggled_cb), app);
+    g_signal_connect (app->monitor_off_radio, "toggled",
+		      G_CALLBACK (monitor_on_off_toggled_cb), app);
+
     app->resolution_combo = glade_xml_get_widget (xml, "resolution_combo");
     g_signal_connect (app->resolution_combo, "changed",
 		      G_CALLBACK (on_resolution_changed), app);
