@@ -21,8 +21,8 @@
 #include "application-tile.h"
 
 #include <string.h>
+#include <glib.h>
 #include <glib/gi18n.h>
-#include <glib/gfileutils.h>
 #include <glib/gstdio.h>
 #include <gconf/gconf-client.h>
 #include <unistd.h>
@@ -150,11 +150,18 @@ application_tile_new_full (const gchar *desktop_item_id,
 
 	desktop_item = load_desktop_item_from_unknown (desktop_item_id);
 
-	if (desktop_item)
+	if (
+		desktop_item &&
+		gnome_desktop_item_get_entry_type (desktop_item) == GNOME_DESKTOP_ITEM_TYPE_APPLICATION
+	)
 		uri = gnome_desktop_item_get_location (desktop_item);
 
-	if (! desktop_item || ! uri)
+	if (! uri) {
+		if (desktop_item)
+			gnome_desktop_item_unref (desktop_item);
+
 		return NULL;
+	}
 
 	this = g_object_new (APPLICATION_TILE_TYPE, "tile-uri", uri, NULL);
 	priv = APPLICATION_TILE_GET_PRIVATE (this);
@@ -323,7 +330,10 @@ application_tile_setup (ApplicationTile *this, const gchar *gconf_prefix)
 	  atk_object_set_description (accessible, desc);
 
 	header    = create_header    (name);
-	if (desc && priv->show_generic_name)  /*if no GenericName then just show and center the Name */
+
+	/*if no GenericName then just show and center the Name */
+	if (desc && priv->show_generic_name
+	    && (!name || strcmp(name, desc) != 0))
 		subheader = create_subheader (desc);
 	else
 		subheader = NULL;
@@ -335,12 +345,12 @@ application_tile_setup (ApplicationTile *this, const gchar *gconf_prefix)
 		"nameplate-image",         image,
 		"nameplate-header",        header,
 		"nameplate-subheader",     subheader,
-		"nameplate-tooltip",	   comment,
 		"context-menu",            context_menu,
 		"application-name",        name,
 		"application-description", desc,
 		"gconf-prefix",            gconf_prefix,
 		NULL);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (this), comment);
 
 	priv->agent = bookmark_agent_get_instance (BOOKMARK_STORE_USER_APPS);
 	g_object_get (G_OBJECT (priv->agent), BOOKMARK_AGENT_STORE_STATUS_PROP, & priv->agent_status, NULL);
